import os
import logging
import colorlog
from pathlib import Path

logger = logging.getLogger()
logger.setLevel(logging.DEBUG)

log_dir = Path("logs/")

if not log_dir.is_dir():
    os.mkdir(log_dir)

debug_log_fp = log_dir / "debug.log"
error_log_fp = log_dir / "error.log"

fh1 = logging.FileHandler(debug_log_fp, mode="a")
fh1.setLevel(logging.DEBUG)
fh2 = logging.FileHandler(error_log_fp, mode="a")
fh2.setLevel(logging.ERROR)
sh = colorlog.StreamHandler()
sh.setLevel(logging.DEBUG)

# Define formatters
formatter = logging.Formatter(
    fmt="%(asctime)s - %(levelname)-8s | %(filename)s:%(lineno)s | %(message)s ",
    datefmt="%Y-%m-%d %H:%M:%S",
)
color_formatter = colorlog.ColoredFormatter(
    fmt="%(log_color)s%(levelname)-8s%(reset)s %(log_color)s%(message)s %(black)s(%(filename)s:%(lineno)s)",
    datefmt=None,
    reset=True,
    log_colors={
        "DEBUG": "green",
        "INFO": "cyan",
        "WARNING": "yellow",
        "ERROR": "red",
        "CRITICAL": "red,bg_yellow",
    },
    secondary_log_colors={},
    style="%",
)

# Set the formatter for each handler
fh1.setFormatter(formatter)
fh2.setFormatter(formatter)
sh.setFormatter(color_formatter)

# Add all three handlers to the logger
logger.addHandler(fh1)
logger.addHandler(fh2)
logger.addHandler(sh)

if __name__ == "__main__":
    logger.debug("This is a debug message")
    logger.info("This is a message for your information")
    logger.warning("This is a warning message")
    logger.error("This is an error message")
    logger.critical("This is a critical error message")



import time


def timeit(func):
    def wrapped_function(*args, **kwargs):
        start_time = time.time()
        result = func(*args, **kwargs)
        elapsed = time.time() - start_time
        return elapsed, result

    return wrapped_function


def timeitprint(func):
    def wrapped_function(*args, **kwargs):
        time_start = time.time()
        result = func(*args, **kwargs)
        time_end = time.time()
        time_elapsed = time_end - time_start
        hours, minutes = 0, 0
        if time_elapsed >= 3600:
            hours = int(time_elapsed // 3600)
            time_elapsed = time_elapsed - hours * 3600
        if time_elapsed >= 60:
            minutes = int(time_elapsed // 60)
            time_elapsed = time_elapsed - minutes * 60
        seconds = round(time_elapsed, 3)
        func_name = func.__name__
        if hours:
            print(
                "====== Func '{}' finished in {} hrs, {} mins, {} secs ======\n".format(
                    func_name, hours, minutes, int(seconds)
                )
            )
        elif minutes:
            print(
                "====== Func '{}' finished in {} mins, {:.2f} secs ======\n".format(
                    func_name, minutes, seconds
                )
            )
        else:
            print(
                "====== Func '{}' finished in {:.10f} secs ======\n".format(
                    func_name, seconds
                )
            )

        return result

    return wrapped_function


if __name__ == "__main__":

    @timeitprint
    def tictok():
        a = 1000000
        for i in range(10000000):
            a -= 1
            b = a
        return

    tictok()



import os
import shutil
from pathlib import Path

from .logger import logger


def safe_rename(src: str, dest: str):
    """
    safety checks:
        1. check if source exists before rename
        2. check if new name destination already exist before rename
    Only rename file when passing above two checks
    """
    src = Path(src)
    dest = Path(dest)

    if not Path(src).exists():
        logger.error(
            f"safe_rename cannot be done, because source file '{src}' not found."
        )
        return
    if Path(dest).exists():
        logger.error(
            f"safe_rename cannot be done, because destination file '{dest}' already exist."
        )
        return
    try:
        os.rename(src, dest)
        logger.debug(f"'{src}' has been renamed to '{dest}'")
    except Exception as err:
        logger.error(f"Rename operation failed, reason: {err}")


def safe_copy(src: str, dest: str):
    """
    safety checks:
        1. check if source exists before copy
        2. check if same-name file already exist in destination before copy
    Only copy file when passing above two checks
    """
    src = Path(src)
    dest = Path(dest)

    if not Path(src).exists():
        logger.error(
            f"safe_copy cannot be done, because source file '{src}' not found."
        )
        return
    if Path(dest).exists():
        logger.warning(
            f"safe_copy cannot be done, because destination file '{dest}' already exist."
        )
        return
    try:
        # subprocess.run(["cp", str(src), str(dest)])
        shutil.copy2(src, dest)
        logger.debug(f"'{src}' has been copied to '{dest}'")
    except Exception as err:
        logger.error(f"Copy operation failed, reason: {err}")


def safe_move(src: str, dest: str):
    """
    safety checks:
        1. check if source exists before move
        2. check if same-name file already exist in destination before move
    Only move file when passing above two checks
    """
    src = Path(src)
    dest = Path(dest)

    if not Path(src).exists():
        logger.error(
            f"safe_move cannot be done, because source file '{src}' not found."
        )
        return
    if Path(dest).exists():
        logger.warning(
            f"safe_move cannot be done, because destination file '{dest}' already exist."
        )
        return
    try:
        os.move(src, dest)
        logger.debug(f"'{src}' has been moved to '{dest}'")
    except Exception as err:
        logger.error(f"Move operation failed, reason: {err}")



from setuptools import setup, find_packages

MAJOR = 0
MINOR = 0
MICRO = 15
VERSION = "%d.%d.%d" % (MAJOR, MINOR, MICRO)

with open("README.md", "r") as f:
    long_description = f.read()

setup(
    name="markkk",
    version=VERSION,
    author="Mark Huang",
    author_email="mark.h.huang@gmail.com",
    description="Python convenient utilities for personal usage",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/MarkHershey/markkk",
    package_dir=({"": "src"}),
    packages=find_packages(where="src"),
    install_requires=["colorlog>=4.1.0"],
    extras_require={"dev": ["pytest", "tox", "wheel"]},
    # Classifiers ref: https://pypi.org/classifiers/
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Topic :: Utilities",
        "Intended Audience :: Developers",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Natural Language :: English",
        "Framework :: tox",
        "Framework :: Pytest",
    ],
    python_requires=">=3.6",
)



import os
from pathlib import Path
from typing import List, Tuple


def is_ascii(text: str) -> bool:
    """ Check if a string contains ascii character only. """
    if isinstance(text, str):
        try:
            text.encode("ascii")
        except UnicodeEncodeError:
            return False
    else:
        try:
            text.decode("ascii")
        except UnicodeDecodeError:
            return False
    return True


def check_non_ascii_index(string: str) -> Tuple[int]:
    if not isinstance(string, str):
        # print(f"{type(string)} is not string.")
        raise TypeError(f"{type(string)} is not string.")

    if is_ascii(string):
        return None

    index_list: List[str] = []

    for index, char in enumerate(string):
        if ord(char) >= 128:
            index_list.append(index + 1)

    return tuple(index_list)


def check_non_ascii_char(string: str) -> Tuple[str]:
    if not isinstance(string, str):
        # print(f"{type(string)} is not string.")
        raise TypeError(f"{type(string)} is not string.")

    if is_ascii(string):
        return None

    index_list = []

    for index, char in enumerate(string):
        if ord(char) >= 128:
            index_list.append(f"{index + 1}: {char}")

    return tuple(index_list)


def is_ascii_only_file(filepath: str) -> bool:
    """ Check if a file contains ascii character only. """
    filepath = Path(filepath)
    assert filepath.is_file()

    with filepath.open(encoding="utf-8") as f:
        text = f.read()

    return is_ascii(text)


def check_file_by_line(filepath: str):
    filepath = Path(filepath)
    assert filepath.is_file()

    if is_ascii_only_file(filepath):
        print("OK")
        return
    else:
        print("\nFollowing lines contain non-ascii characters:\n")

    with filepath.open(encoding="utf-8") as f:
        lines = f.readlines()

    for index, line in enumerate(lines):
        line_number = index + 1
        if not is_ascii(line):
            print(f"{line_number}: {check_non_ascii_char(line)}")

    print("---END---\n")


def ensure_no_zh_punctuation(string: str) -> str:
    """Convert common Chinese (zh) punctuation (Unicode) to
    corresponding English (en-us) punctuation (ascii).
    """
    if not isinstance(string, str):
        print(f"{type(string)} is not string.")

    if is_ascii(string):
        return string

    punc_map = {
        "。": ".",
        "，": ",",
        "、": ",",
        "？": "?",
        "！": "!",
        "；": ";",
        "：": ":",
        "‘": "'",
        "’": "'",
        "“": '"',
        "”": '"',
        "【": "[",
        "】": "]",
        "「": "{",
        "」": "}",
        "｜": "|",
        "—": "-",  # ord = 8212
        "（": "(",
        "）": ")",
        "～": "~",
        "《": "<",
        "》": ">",
        "……": "...",
        "…": "...",
        "·": "-",
        "¥": "$",
    }
    new_string = ""
    for char in string:
        # print(f"{char}: {ord(char) if ord(char) >= 128 else '>>>ascii'}")
        new_string += punc_map.get(char, char)
    return new_string


def replace_punc_for_file(filepath: str) -> Path:
    filepath = Path(filepath)
    assert filepath.is_file()
    filepath = filepath.resolve()

    head, tail = os.path.split(filepath)
    name, ext = os.path.splitext(tail)

    with filepath.open(encoding="utf-8") as f:
        content = f.read()

    content_new = ensure_no_zh_punctuation(content)
    filepath_new = Path(head) / f"{name}_new{ext}"

    with filepath_new.open(mode="w", encoding="utf-8") as f:
        f.write(content_new)
        print(f"Successfully exported: {filepath_new}")

    return filepath_new


if __name__ == "__main__":
    # ensure_no_zh_punctuation("哈水淀粉和卡积分")
    # ensure_no_zh_punctuation("，。？！#¥%……&*（）——-=+【】「」；‘：“《》/”’")
    replace_punc_for_file("test.txt")
